//
// exercisetablerow -- Exercise all aspects of the Fl_Table_Row widget
//

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Check_Button.H>
#include <FL/fl_draw.H>
#include <FL/fl_ask.H>

#include "Fl_Table_Row.H"

#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <algorithm>		// STL sort

//
// Simple demonstration class to derive from Fl_Table_Row
//
class DemoTable : public Fl_Table_Row
{
protected:
    void draw_cell(TableContext context,  		// table cell drawing
    		   int R=0, int C=0, int X=0, int Y=0, int W=0, int H=0);
    void callback(TableContext context, 		// callback for table events
    		   int R, int C);

public:
    DemoTable(int x, int y, int w, int h, const char *l=0) : Fl_Table_Row(x,y,w,h,l)
	{ }
    ~DemoTable() { }
};

// Handle drawing all cells in table
void DemoTable::draw_cell(TableContext context, 
			  int R=0, int C=0, int X=0, int Y=0, int W=0, int H=0)
{
    static char s[40];
    sprintf(s, "%d/%d", R, C);		// text for each cell

    switch ( context )
    {
	case CONTEXT_NEWPAGE:
	    fl_font(FL_HELVETICA, 16);
	    return;

	case CONTEXT_COL_HEADER:
	    fl_push_clip(X, Y, W, H);
	    {
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, color());
		fl_color(FL_BLACK);
		fl_draw(s, X, Y, W, H, FL_ALIGN_CENTER);
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_CELL:
	{
	    fl_push_clip(X, Y, W, H);
	    {
	        // BG COLOR
		fl_color( row_selected(R) ? selection_color() : FL_WHITE);
		fl_rectf(X, Y, W, H);

		// TEXT
		fl_color(FL_BLACK);
		fl_draw(s, X, Y, W, H, FL_ALIGN_CENTER);

		// BORDER
		fl_color(FL_LIGHT2); 
		fl_rect(X, Y, W, H);
	    }
	    fl_pop_clip();
	    return;
	}
    }
}

// Callback whenever someone clicks on different parts of the table
void DemoTable::callback(TableContext context, int R, int C)
{
    cerr << "DemoTable callback: context is " << (int)context << endl;
}

DemoTable *G_table = 0;

void setrows_cb(Fl_Widget*, void *data)
{
    Fl_Input *in = (Fl_Input*)data;
    int rows = atoi(in->value());
    if ( rows < 0 ) rows = 0;
    G_table->rows(rows);
}

void setcols_cb(Fl_Widget*, void *data)
{
    Fl_Input *in = (Fl_Input*)data;
    int cols = atoi(in->value());
    if ( cols < 0 ) cols = 0;
    G_table->cols(cols);
}

void setrowheader_cb(Fl_Widget*, void *data)
{
    Fl_Check_Button *check = (Fl_Check_Button*)data;
    G_table->row_header(check->value());
}

void setcolheader_cb(Fl_Widget*, void *data)
{
    Fl_Check_Button *check = (Fl_Check_Button*)data;
    G_table->col_header(check->value());
}

void setrowresize_cb(Fl_Widget*, void *data)
{
    Fl_Check_Button *check = (Fl_Check_Button*)data;
    G_table->row_resize(check->value());
}

void setcolresize_cb(Fl_Widget*, void *data)
{
    Fl_Check_Button *check = (Fl_Check_Button*)data;
    G_table->col_resize(check->value());
}

void settoprow_cb(Fl_Widget *w, void *data)
{
    Fl_Input *in = (Fl_Input*)data;
    int toprow = atoi(in->value());
    if ( toprow < 0 || toprow >= G_table->rows() ) 
        { fl_alert("Must be in range 0 thru #rows"); }
    else
        { G_table->top_row(toprow); }
}

char *itoa(int val)
{
    static char s[80];
    sprintf(s, "%d", val);
    return(s);
}

int main()
{
    Fl_Window win(900, 700);

    G_table = new DemoTable(20, 20, 860, 460);
    G_table->col_header(1);
    G_table->col_resize(1);
    G_table->selection_color(FL_YELLOW);
    G_table->when(FL_WHEN_RELEASE);
    G_table->rows(100);
    G_table->cols(50);
    G_table->col_header(1);
    G_table->col_resize(1);

    Fl_Input setrows(100, 500, 120, 25, "Rows");
    setrows.value(itoa(G_table->rows()));
    setrows.callback(setrows_cb, (void*)&setrows);
    setrows.when(FL_WHEN_RELEASE);

    Fl_Input setcols(100, 530, 120, 25, "Cols");
    setcols.value(itoa(G_table->cols()));
    setcols.callback(setcols_cb, (void*)&setcols);
    setcols.when(FL_WHEN_RELEASE);

    Fl_Check_Button rowheader(100, 570, 120, 25, "Row Headers?");
    rowheader.callback(setrowheader_cb, (void*)&rowheader);
    rowheader.value(1);
    rowheader.deactivate();		// not supported

    Fl_Check_Button colheader(250, 570, 120, 25, "Col Headers?");
    colheader.callback(setcolheader_cb, (void*)&colheader);
    colheader.value(1);

    Fl_Check_Button rowresize(100, 600, 120, 25, "Row Resize?");
    rowresize.callback(setrowresize_cb, (void*)&rowresize);
    rowresize.value(1);
    rowresize.deactivate();		// not supported

    Fl_Check_Button colresize(250, 600, 120, 25, "Col Resize?");
    colresize.callback(setcolresize_cb, (void*)&colresize);
    colresize.value(1);

    Fl_Input toprow(100, 630, 120, 25, "Top Row");
    toprow.value("1");
    toprow.callback(settoprow_cb, (void*)&toprow);
    toprow.when(FL_WHEN_RELEASE);

    // todo: select rows

    win.end();
    win.resizable(*G_table);
    win.show();
    return(Fl::run());
}
