//
// testtablerow -- Simple test of Fl_Table_Row class
//

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/fl_draw.H>

#include "Fl_Table_Row.H"

#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <algorithm>		// STL sort

//
// Simple demonstration class to derive from Fl_Table_Row
//
class DemoTable : public Fl_Table_Row
{
protected:
    void draw_cell(TableContext context,  		// table cell drawing
    		   int R=0, int C=0, int X=0, int Y=0, int W=0, int H=0);
    void callback(TableContext context, 		// callback for table events
    		   int R, int C);

public:
    DemoTable(int x, int y, int w, int h, const char *l=0) : Fl_Table_Row(x,y,w,h,l)
	{ }
    ~DemoTable() { }
};

// Handle drawing all cells in table
void DemoTable::draw_cell(TableContext context, 
			  int R=0, int C=0, int X=0, int Y=0, int W=0, int H=0)
{
    static char s[40];
    sprintf(s, "%d/%d", R, C);		// text for each cell

    switch ( context )
    {
	case CONTEXT_NEWPAGE:
	    fl_font(FL_HELVETICA, 16);
	    return;

	case CONTEXT_COL_HEADER:
	    fl_push_clip(X, Y, W, H);
	    {
		fl_draw_box(FL_THIN_UP_BOX, X, Y, W, H, color());
		fl_color(FL_BLACK);
		fl_draw(s, X, Y, W, H, FL_ALIGN_CENTER);
	    }
	    fl_pop_clip();
	    return;

	case CONTEXT_CELL:
	{
	    fl_push_clip(X, Y, W, H);
	    {
	        // BG COLOR
		fl_color( row_selected(R) ? selection_color() : FL_WHITE);
		fl_rectf(X, Y, W, H);

		// TEXT
		fl_color(FL_BLACK);
		fl_draw(s, X, Y, W, H, FL_ALIGN_CENTER);

		// BORDER
		fl_color(FL_LIGHT2); 
		fl_rect(X, Y, W, H);
	    }
	    fl_pop_clip();
	    return;
	}
    }
}

// Callback whenever someone clicks on different parts of the table
void DemoTable::callback(TableContext context, int R, int C)
{
    cerr << "DemoTable callback: context is " << (int)context << endl;
}

int main()
{
    Fl_Window win(900,500);
    DemoTable table(20, 20, win.w()-40, win.h()-40);
    table.col_header(1);
    table.col_resize(1);
    table.selection_color(FL_YELLOW);
    table.when(FL_WHEN_RELEASE);	// handle table events on release
    table.rows(1001);
    table.cols(31);
    table.col_header(1);		// enable col header
    table.col_resize(1);		// enable col resizing
    win.end();
    win.resizable(table);
    win.show();
    return(Fl::run());
}

